@@ -1,4 +1,4 @@
-/* $Id: acl.c,v 1.106 2013/01/19 16:01:15 manu Exp $ */
+/* $Id: acl.c,v 1.115 2015/10/26 05:29:58 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2012 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: acl.c,v 1.106 2013/01/19 16:01:15 manu Exp $");
+__RCSID("$Id: acl.c,v 1.115 2015/10/26 05:29:58 manu Exp $");
 #endif
 #endif
 
@@ -79,9 +79,7 @@ __RCSID("$Id: acl.c,v 1.106 2013/01/19 16:01:15 manu Exp $");
 #ifdef USE_LDAP
 #include "ldapcheck.h"
 #endif
-#if defined(USE_CURL) || defined(USE_LDAP)
 #include "prop.h"
-#endif
 #ifdef USE_GEOIP
 #include "geoip.h"
 #endif
@@ -155,14 +153,24 @@ char *acl_print_urlcheck(acl_data_t *, char *, size_t);
 void acl_add_ldapcheck(acl_data_t *, void *);
 char *acl_print_ldapcheck(acl_data_t *, char *, size_t);
 #endif
-#if defined(USE_CURL) || defined(USE_LDAP)
 void acl_add_prop_string(acl_data_t *, void *);
+void acl_add_prop_pop(acl_data_t *, void *);
+void acl_add_prop_opnum(acl_data_t *, void *);
 void acl_add_prop_regex(acl_data_t *, void *);
 char *acl_print_prop_string(acl_data_t *, char *, size_t);
+char *acl_print_prop_pop(acl_data_t *, char *, size_t);
+char *acl_print_prop_opnum(acl_data_t *, char *, size_t);
 char *acl_print_prop_regex(acl_data_t *, char *, size_t);
 void acl_free_prop_string(acl_data_t *);
+void acl_free_prop_pop(acl_data_t *);
+void acl_free_prop_opnum(acl_data_t *);
 void acl_free_prop_regex(acl_data_t *);
+#ifdef USE_NSUPDATE
+char *acl_print_nsupdate(acl_data_t *, char *, size_t);
+void acl_add_nsupdate(acl_data_t *, void *);
 #endif
+int acl_log_string(acl_data_t *, acl_stage_t, 
+		   struct acl_param *, struct mlfi_priv *);
 
 struct acl_clause_rec acl_clause_rec[] = {
 	/* Temporary types for lists */
@@ -252,12 +260,10 @@ struct acl_clause_rec acl_clause_rec[] = {
 	  AT_LIST, AC_NONE, AC_NONE, EXF_BODY,
 	  acl_print_list, acl_add_list, 
 	  NULL, acl_list_filter },
-#if defined(USE_CURL) || defined(USE_LDAP)
 	{ AC_BODY_PROP, MULTIPLE_OK, AS_DATA, "body_prop", 
 	  AT_STRING, AC_BODY_LIST, AC_STRING, EXF_BODY,
 	  acl_print_string, acl_add_body_string, 
 	  acl_free_string, prop_body_validate },
-#endif
 	{ AC_HEADER, MULTIPLE_OK, AS_DATA, "header", 
 	  AT_STRING, AC_HEADER_LIST, AC_STRING, EXF_HEADER,
 	  acl_print_string, acl_add_body_string, 
@@ -270,12 +276,10 @@ struct acl_clause_rec acl_clause_rec[] = {
 	  AT_LIST, AC_NONE, AC_NONE, EXF_HEADER,
 	  acl_print_list, acl_add_list, 
 	  NULL, acl_list_filter },
-#if defined(USE_CURL) || defined(USE_LDAP)
 	{ AC_HEADER_PROP, MULTIPLE_OK, AS_DATA, "header_prop", 
 	  AT_STRING, AC_HEADER_LIST, AC_STRING, EXF_HEADER,
 	  acl_print_string, acl_add_body_string, 
 	  acl_free_string, prop_header_validate },
-#endif
 	{ AC_MACRO, MULTIPLE_OK, AS_ANY, "macro", 
 	  AT_MACRO, AC_MACRO_LIST, AC_STRING, EXF_MACRO,
 	  acl_print_macro, acl_add_macro,
@@ -320,16 +324,26 @@ struct acl_clause_rec acl_clause_rec[] = {
 	  acl_print_ldapcheck, acl_add_ldapcheck,
 	  NULL, ldapcheck_validate },
 #endif
-#if defined(USE_CURL) || defined(USE_LDAP)
-	{ AC_PROP, MULTIPLE_OK, AS_ANY, "prop", 
-	  AT_PROP, AC_NONE, AC_PROP, EXF_PROP,
+	{ AC_PROP_STR, MULTIPLE_OK, AS_ANY, "prop", 
+	  AT_PROP, AC_NONE, AC_PROP_STR, EXF_PROP,
 	  acl_print_prop_string, acl_add_prop_string,
 	  acl_free_prop_string, prop_string_validate },
+	{ AC_PROP_GLOB, MULTIPLE_OK, AS_ANY, "glob_prop", 
+	  AT_PROP, AC_NONE, AC_PROP_GLOB, EXF_PROP,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_glob_validate },
+	{ AC_PROP_PROP, MULTIPLE_OK, AS_ANY, "propprop", 
+	  AT_PROP, AC_NONE, AC_PROP_PROP, EXF_PROP,
+	  acl_print_prop_pop, acl_add_prop_pop,
+	  acl_free_prop_pop,  prop_pop_validate },
+	{ AC_PROP_NUM, MULTIPLE_OK, AS_ANY, "prop_num", 
+	  AT_PROP, AC_NONE, AC_PROP_NUM, EXF_PROP,
+	  acl_print_prop_opnum, acl_add_prop_opnum,
+	  acl_free_prop_opnum, prop_lhsnum_validate },
 	{ AC_PROP_RE, MULTIPLE_OK, AS_ANY, "prop_re", 
 	  AT_PROP, AC_NONE, AC_PROP_RE, EXF_PROP,
 	  acl_print_prop_regex, acl_add_prop_regex,
 	  acl_free_prop_regex, prop_regex_validate },
-#endif
 	{ AC_AUTH, MULTIPLE_OK, AS_ANY, "auth", 
 	  AT_STRING, AC_AUTH_LIST, AC_STRING, EXF_AUTH,
 	  acl_print_string, acl_add_string,
@@ -371,22 +385,18 @@ struct acl_clause_rec acl_clause_rec[] = {
 	  AT_OPNUM, AC_NONE, AC_MSGSIZE, EXF_MSGSIZE,
 	  acl_print_opnum, acl_add_opnum,
 	  NULL, acl_msgsize_cmp },
-#if defined(USE_CURL) || defined(USE_LDAP)
 	{ AC_MSGSIZE_PROP, MULTIPLE_OK, AS_DATA, "msgsize_prop", 
 	  AT_OPNUM, AC_NONE, AC_STRING_PROP, EXF_MSGSIZE,
 	  prop_opnum_print, prop_opnum_add,
-	  prop_opnum_free, prop_opnum_validate },
-#endif
+	  prop_opnum_free, prop_rhsnum_validate },
 	{ AC_RCPTCOUNT, MULTIPLE_OK, AS_ANY, "rcptcount", 
 	  AT_OPNUM, AC_NONE, AC_RCPTCOUNT, EXF_RCPTCOUNT,
 	  acl_print_opnum, acl_add_opnum_body,
 	  NULL, acl_rcptcount_cmp },
-#if defined(USE_CURL) || defined(USE_LDAP)
 	{ AC_RCPTCOUNT_PROP, MULTIPLE_OK, AS_DATA, "rcptcount_prop", 
 	  AT_OPNUM, AC_NONE, AC_STRING_PROP, EXF_RCPTCOUNT,
 	  acl_print_prop_string, acl_add_prop_string,
-	  acl_free_prop_string, prop_opnum_validate },
-#endif
+	  acl_free_prop_string, prop_rhsnum_validate },
 	{ AC_CLOCKSPEC, MULTIPLE_OK, AS_ANY, "time",
 	  AT_CLOCKSPEC, AC_NONE, AC_CLOCKSPEC, EXF_CLOCKSPEC,
 	  print_clockspec, add_clockspec,
@@ -426,12 +436,10 @@ struct acl_clause_rec acl_clause_rec[] = {
 	{ AC_SASCORE, MULTIPLE_OK, AS_DATA, "spamd score",
 	  AT_OPNUM, AC_NONE, AC_NONE,  EXF_SA,
 	  acl_print_opnum, acl_add_opnum, NULL, spamd_score },
-#if defined(USE_CURL) || defined(USE_LDAP)
 	{ AC_SASCORE_PROP, MULTIPLE_OK, AS_DATA, "sascore_prop", 
 	  AT_OPNUM, AC_NONE, AC_STRING_PROP, EXF_SA,
 	  acl_print_prop_string, acl_add_prop_string,
-	  acl_free_prop_string, prop_opnum_validate },
-#endif /* USE_CURL || USE_LDAP */
+	  acl_free_prop_string, prop_rhsnum_validate },
 #endif /* USE_SPAMD */
 #ifdef HAVE_DATA_CALLBACK
 	{ AC_TARPIT, UNIQUE, AS_ANY, "tarpit",
@@ -439,6 +447,64 @@ struct acl_clause_rec acl_clause_rec[] = {
 	  acl_print_time, acl_add_time,
 	  NULL, acl_tarpit_filter },
 #endif
+#ifdef USE_NSUPDATE
+	{ AC_NSUPDATE, MULTIPLE_OK, AS_ANY, "nsupdate",
+	  AT_NSUPDATE, AC_NONE, AC_STRING, EXF_NONE,
+	  acl_print_nsupdate, acl_add_nsupdate,
+	  NULL, nsupdate_filter },
+#endif
+	{ AC_EQSET, MULTIPLE_OK, AS_ANY, "set", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_eqset_string },
+	{ AC_EQRSET, MULTIPLE_OK, AS_ANY, "rset", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_eqrset_string },
+	{ AC_INCSET, MULTIPLE_OK, AS_ANY, "incset", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_incset_string },
+	{ AC_INCRSET, MULTIPLE_OK, AS_ANY, "incrset", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_incrset_string },
+	{ AC_DECSET, MULTIPLE_OK, AS_ANY, "decset", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_decset_string },
+	{ AC_DECRSET, MULTIPLE_OK, AS_ANY, "decrset", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_decrset_string },
+	{ AC_EQSETPROP, MULTIPLE_OK, AS_ANY, "setprop", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_eqset_prop },
+	{ AC_EQRSETPROP, MULTIPLE_OK, AS_ANY, "rsetprop", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_eqrset_prop },
+	{ AC_INCSETPROP, MULTIPLE_OK, AS_ANY, "incsetprop", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_incset_prop },
+	{ AC_INCRSETPROP, MULTIPLE_OK, AS_ANY, "incrsetprop", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_incrset_prop },
+	{ AC_DECSETPROP, MULTIPLE_OK, AS_ANY, "decsetprop", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_decset_prop },
+	{ AC_DECRSETPROP, MULTIPLE_OK, AS_ANY, "decrsetprop", 
+	  AT_PROP, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_prop_string, acl_add_prop_string,
+	  acl_free_prop_string, prop_decrset_prop },
+	{ AC_LOG, MULTIPLE_OK, AS_ANY, "log", 
+	  AT_STRING, AC_NONE, AC_NONE, EXF_NONE,
+	  acl_print_string, acl_add_string,
+	  acl_free_string, acl_log_string },
 };
 
 struct {
@@ -845,12 +911,45 @@ acl_helo_strstr(ad, stage, ap, priv)
 	struct acl_param *ap;
 	struct mlfi_priv *priv;
 {
-	char *helo = ad->string;
+	char *test_helo;
+	char lh_priv[ADDRLEN + 1];
+	char lh_test[ADDRLEN + 1];
+	int i, res;
 
-	printf("-> %s/%s\n", priv->priv_helo, helo);
-	if (strstr(priv->priv_helo, helo) != NULL) 
-		return 1;
-	return 0;
+	res = 0;
+	test_helo = fstring_expand(priv, NULL, ad->string, NULL);
+
+	/* Convert domain-names (ASCII) to same-case for comparison */
+	for (i = 0; i < sizeof(lh_priv) && priv->priv_helo[i]; i++)
+		lh_priv[i] = tolower((int)(priv->priv_helo[i]));
+	lh_priv[i] = '\0';
+
+	for (i = 0; i < sizeof(lh_test) && test_helo[i]; i++)
+		lh_test[i] = tolower((int)(test_helo[i]));
+	lh_test[i] = '\0';
+
+	/*
+	 * Note that for historical reasons (maybe considered a bug,
+	 * but maybe some deployments now rely on this behavior)
+	 * strstr() is used rather than strcmp(), allowing substring
+	 * matches to be treated as good hits.
+	 * Earlier versions did only the strstr(priv_helo,test_helo).
+	 */
+	if ((strstr(lh_priv, lh_test) != NULL) ||
+	    (strstr(lh_test, lh_priv) != NULL)) 
+		res = 1;
+
+	if (conf.c_debug)
+		mg_log(LOG_DEBUG,
+			"acl_helo_strstr: \"%s\"/\"%s\" "
+			"(lowercase of \"%s\"/\"%s\") => %s\n",
+			lh_priv, lh_test,
+			priv->priv_helo, test_helo,
+			res ? "MATCHED" : "MISMATCH");
+
+	free(test_helo);
+
+	return res;
 }
 
 int
@@ -1052,11 +1151,9 @@ acl_print_null(ad, buf, len)
 	return buf;
 }
 
-char *
-acl_print_opnum(ad, buf, len)
-	acl_data_t *ad;
-	char *buf;
-	size_t len;
+static char *
+acl_print_op(op)
+	enum operator op;
 {
 	struct {
 		enum operator op;
@@ -1070,20 +1167,24 @@ acl_print_opnum(ad, buf, len)
 		{ OP_LE, "<=" },
 	};
 	int i;
-	char *str = NULL;
 
 	for (i = 0; i < sizeof(op_to_str) / sizeof(*op_to_str); i++) {
-		if (op_to_str[i].op == ad->opnum.op) {
-			str = op_to_str[i].str;
-			break;
-		}
-	}
-	if (str == NULL) {
-		mg_log(LOG_ERR, "unexpected operator");
-		exit(EX_SOFTWARE);
+		if (op_to_str[i].op == op)
+			return op_to_str[i].str;
 	}
 
-	snprintf(buf, len, "%s %d", str, ad->opnum.num);
+	mg_log(LOG_ERR, "unexpected operator");
+	exit(EX_SOFTWARE);
+}
+
+
+char *
+acl_print_opnum(ad, buf, len)
+	acl_data_t *ad;
+	char *buf;
+	size_t len;
+{
+	snprintf(buf, len, "%s %d", acl_print_op(ad->opnum.op), ad->opnum.num);
 
 	return buf;
 }
@@ -1566,7 +1667,6 @@ acl_print_ldapcheck(ad, buf, len)
 }
 #endif
 
-#if defined(USE_CURL) || defined(USE_LDAP)
 void
 acl_add_prop_string(ad, data)
 	acl_data_t *ad;
@@ -1596,6 +1696,62 @@ acl_add_prop_string(ad, data)
 
 	return;
 }
+
+void
+acl_add_prop_pop(ad, data)
+	acl_data_t *ad;
+	void *data;
+{
+	struct acl_prop_pop *apop = (struct acl_prop_pop *)data;
+
+	if ((ad->apop = malloc(sizeof(*ad->apop))) == NULL) {
+		mg_log(LOG_ERR, "acl malloc failed: %s", strerror(errno));
+		exit(EX_OSERR);
+	}
+
+	if ((ad->apop->apop_rhs = strdup(apop->apop_rhs)) == NULL) {
+		mg_log(LOG_ERR, "acl strdup failed: %s", strerror(errno));
+		exit(EX_OSERR);
+	}
+
+	ad->apop->apop_op = apop->apop_op;
+
+	if ((ad->apop->apop_lhs = strdup(apop->apop_lhs)) == NULL) {
+		mg_log(LOG_ERR, "acl strdup failed: %s", strerror(errno));
+		exit(EX_OSERR);
+	}
+}
+
+void
+acl_add_prop_opnum(ad, data)
+	acl_data_t *ad;
+	void *data;
+{
+	struct prop_data *upd;
+
+	upd = (struct prop_data *)data;
+
+	if ((ad->prop = malloc(sizeof(*ad->prop))) == NULL) {
+		mg_log(LOG_ERR, "acl malloc failed: %s", strerror(errno));
+		exit(EX_OSERR);
+	}
+
+	if ((ad->prop->upd_name = strdup(upd->upd_name + 1)) == NULL) {
+		mg_log(LOG_ERR, "acl strdup failed: %s", strerror(errno));
+		exit(EX_OSERR);
+	}
+
+	if ((ad->prop->upd_data = malloc(sizeof(acl_data_t))) == NULL) {
+		mg_log(LOG_ERR, "acl malloc failed: %s", strerror(errno));
+		exit(EX_OSERR);
+	}
+	memset(ad->prop->upd_data, 0, sizeof(acl_data_t));
+
+	acl_add_opnum((void *)ad->prop->upd_data, upd->upd_data);
+
+	return;
+}
+
 void 
 acl_add_prop_regex(ad, data)
 	acl_data_t *ad;
@@ -1640,6 +1796,35 @@ acl_print_prop_string(ad, buf, len)
 	return buf;
 }
 
+char *
+acl_print_prop_pop(ad, buf, len)
+	acl_data_t *ad;
+	char *buf;
+	size_t len;
+{
+	snprintf(buf, len, "$%s %s $%s ",
+		 ad->apop->apop_lhs,
+		 acl_print_op(ad->apop->apop_op),
+		 ad->apop->apop_rhs);
+
+	return buf;
+}
+
+char *
+acl_print_prop_opnum(ad, buf, len)
+	acl_data_t *ad;
+	char *buf;
+	size_t len;
+{
+	size_t written;
+
+	written = snprintf(buf, len, "$%s ", ad->prop->upd_name);
+	acl_print_opnum(ad->prop->upd_data, buf + written, len - written);
+
+	return buf;
+}
+
+
 char *
 acl_print_prop_regex(ad, buf, len)
 	acl_data_t *ad;
@@ -1665,6 +1850,29 @@ acl_free_prop_string(ad)
 
 	return;
 }
+
+void 
+acl_free_prop_opnum(ad)
+	acl_data_t *ad;
+{
+	free(ad->prop->upd_data);
+	free(ad->prop->upd_name);
+	free(ad->prop);
+
+	return;
+}
+
+void 
+acl_free_prop_pop(ad)
+	acl_data_t *ad;
+{
+	free(ad->apop->apop_rhs);
+	free(ad->apop->apop_lhs);
+	free(ad->apop);
+
+	return;
+}
+
 void 
 acl_free_prop_regex(ad)
 	acl_data_t *ad;
@@ -1677,6 +1885,26 @@ acl_free_prop_regex(ad)
 	return;
 }
 
+#ifdef USE_NSUPDATE
+void
+acl_add_nsupdate(ad, data)
+	acl_data_t *ad;
+	void *data;
+{
+	ad->nsupdate = (struct nsupdate_entry *)data;
+}
+
+char *
+acl_print_nsupdate(ad, buf, len)
+	acl_data_t *ad;
+	char *buf;
+	size_t len;
+{
+	(void)snprintf(buf, len, "$%s ", ad->nsupdate->nse_name);
+
+	return buf;
+}
+
 #endif
 
 void
@@ -2045,6 +2273,7 @@ acl_actions(priv, stage, acl, ap, retval)
 			mg_log(LOG_ERR, "strdup failed");
 			exit(EX_OSERR);
 		}
+		priv->priv_sr.sr_addheader_index = ap->ap_addheader_index;
 	}
 	if (ap->ap_addfooter) {
 		priv->priv_sr.sr_addfooter = strdup(ap->ap_addfooter);
@@ -2168,6 +2397,11 @@ acl_filter(stage, ctx, priv)
 		if (acl->a_stage != stage)
 			continue;
 
+		/*
+		 * Keep track of current ACL for %cA and %ca format strings
+		 */
+		priv->priv_acl = acl;
+
 		retval = 0;
 		noretval = 0;
 		found = -1;
@@ -2185,6 +2419,7 @@ acl_filter(stage, ctx, priv)
 		ap.ap_msg = acl->a_msg;
 		ap.ap_report = acl->a_report;
 		ap.ap_addheader = acl->a_addheader;
+		ap.ap_addheader_index = acl->a_addheader_index;
 		ap.ap_addfooter = acl->a_addfooter;
 		ap.ap_subjtag = acl->a_subjtag;
 		ap.ap_maxpeek = acl->a_maxpeek;
@@ -2557,8 +2792,13 @@ acl_entry(entrystr, len, acl)
 		mystrlcat(entrystr, tempstr, len);
 	}
 	if (acl->a_addheader) {
-		snprintf(tempstr, sizeof(tempstr), 
-		    "[addheader \"%s\"] ", acl->a_addheader);
+		if (acl->a_addheader_index >= 0)
+			snprintf(tempstr, sizeof(tempstr), 
+				"[addheader \"%s\",%d] ",
+				acl->a_addheader, acl->a_addheader_index);
+		else
+			snprintf(tempstr, sizeof(tempstr), 
+				"[addheader \"%s\"] ", acl->a_addheader);
 		mystrlcat(entrystr, tempstr, len);
 	}
 	if (acl->a_addfooter) {
@@ -2837,8 +3077,9 @@ acl_add_report(report)
 }
 
 void 
-acl_add_addheader(hdr)
+acl_add_addheader(hdr, index)
 	char *hdr;
+	int index;
 {
 	if (gacl->a_addheader) {
 		mg_log(LOG_ERR,
@@ -2851,9 +3092,11 @@ acl_add_addheader(hdr)
 		    "malloc failed in ACL line %d", conf_line);
 		exit(EX_OSERR);
 	}
+
+	gacl->a_addheader_index = index;
 		
 	if (conf.c_debug || conf.c_acldebug)
-		mg_log(LOG_DEBUG, "load acl addheader \"%s\"", hdr);
+		mg_log(LOG_DEBUG, "load acl addheader \"%s\",%d", hdr, index);
 
 	return;
 }
@@ -3031,3 +3274,20 @@ out:
 
 	return 0;
 }
+
+int 
+acl_log_string(ad, stage, ap, priv)
+	acl_data_t *ad; 
+	acl_stage_t stage; 
+	struct acl_param *ap;
+	struct mlfi_priv *priv;
+{
+	char *string;
+
+	string = fstring_expand(priv, NULL, ad->string, NULL);
+	mg_log(LOG_INFO, string);	
+	free(string);
+
+	/* always match */
+	return 1;
+}
