@@ -1,4 +1,4 @@
-/* $Id: acl.h,v 1.51 2013/01/19 16:01:15 manu Exp $ */
+/* $Id: acl.h,v 1.56 2015/10/05 04:18:33 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2012 Emmanuel Dreyfus
@@ -52,7 +52,7 @@ typedef enum { A_GREYLIST, A_WHITELIST, A_BLACKLIST, A_CONTINUE, } acl_type_t;
 typedef enum { AS_NONE, AS_RCPT, AS_DATA, AS_ANY, } acl_stage_t;
 typedef enum { AT_NONE, AT_STRING, AT_REGEX, AT_NETBLOCK, AT_OPNUM, 
 	       AT_CLOCKSPEC, AT_DNSRBL, AT_MX, AT_URLCHECK, AT_MACRO, 
-	       AT_LIST, AT_PROP, AT_SPF, AT_DKIM, 
+	       AT_LIST, AT_PROP, AT_SPF, AT_DKIM, AT_NSUPDATE,
 	       AT_LDAPCHECK, AT_TIME, AT_RATELIMIT } acl_data_type_t;
 
 typedef enum {
@@ -93,7 +93,10 @@ typedef enum {
 	AC_URLCHECK,
 	AC_URLCHECK_LIST,
 	AC_LDAPCHECK,
-	AC_PROP,
+	AC_PROP_STR,
+	AC_PROP_GLOB,
+	AC_PROP_NUM,
+	AC_PROP_PROP,
 	AC_PROP_RE,
 	AC_HEADERPROP,
 	AC_AUTH,
@@ -120,13 +123,31 @@ typedef enum {
 	AC_SASCORE_PROP,
 	AC_TARPIT,
 	AC_RATELIMIT,
+	AC_NSUPDATE,
+	AC_EQSET,
+	AC_EQRSET,
+	AC_INCSET,
+	AC_INCRSET,
+	AC_DECSET,
+	AC_DECRSET,
+	AC_EQSETPROP,
+	AC_EQRSETPROP,
+	AC_INCSETPROP,
+	AC_INCRSETPROP,
+	AC_DECSETPROP,
+	AC_DECRSETPROP,
+	AC_LOG,
 } acl_clause_t;
 
+typedef union acl_data acl_data_t;
 struct acl_clause;
 struct acl_param;
 
 #include "pending.h"
 #include "milter-greylist.h"
+#ifdef USE_NSUPDATE
+#include "nsupdate.h"
+#endif
 
 #define ACL_WRLOCK WRLOCK(acl_lock) 
 #define ACL_RDLOCK RDLOCK(acl_lock) 
@@ -162,6 +183,7 @@ struct acl_param {
 	char *ap_msg;
 	char *ap_report;
 	char *ap_addheader;
+	int ap_addheader_index;
 	char *ap_addfooter;
 	char *ap_subjtag;
 	int ap_maxpeek;
@@ -184,7 +206,7 @@ struct acl_param {
 
 struct all_list_entry;
 
-typedef union acl_data {
+union acl_data {
 	struct acl_netblock_data netblock;
 	char *string;
 	struct {
@@ -206,10 +228,9 @@ typedef union acl_data {
 #ifdef USE_LDAP
 	struct ldapcheck_entry *ldapcheck;
 #endif
-#if defined(USE_CURL) || defined(USE_LDAP)
 	struct prop_data *prop;
 	struct acl_opnum_prop *aonp;
-#endif
+	struct acl_prop_pop *apop;
 	struct acl_opnum_data opnum;
 	struct clockspec *clockspec;
 #if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
@@ -221,7 +242,10 @@ typedef union acl_data {
 #endif
 	time_t time;
 	struct ratelimit_conf *ratelimit_conf;
-} acl_data_t;
+#ifdef USE_NSUPDATE
+	struct nsupdate_entry *nsupdate;
+#endif
+};
 
 struct acl_clause_rec {
 	acl_clause_t acr_type;
@@ -267,6 +291,7 @@ struct acl_entry {
 	char *a_msg;
 	char *a_report;
 	char *a_addheader;
+	int a_addheader_index;
 	char *a_addfooter;
 	char *a_subjtag;
 	int a_maxpeek;
@@ -294,7 +319,7 @@ void acl_add_code(char *);
 void acl_add_ecode(char *);
 void acl_add_msg(char *);
 void acl_add_report(char *);
-void acl_add_addheader(char *);
+void acl_add_addheader(char *, int);
 void acl_add_addfooter(char *);
 void acl_add_subjtag(char *);
 void acl_add_maxpeek(int);
@@ -384,9 +409,9 @@ int acl_modify_by_prop(char *, char *, struct acl_param *);
 #define EXF_HELO	(1LL << 26)
 #define EXF_LDAPCHECK	(1LL << 27)
 #define EXF_NOLOG	(1LL << 28)
-#define	EXF_DKIM	(1LL << 28)
 #define EXF_P0F		(1LL << 29)
 #define EXF_SA		(1LL << 30)
 #define EXF_TARPIT	(1LL << 31)
 #define EXF_MX		(1LL << 32) /* Need long long (64b) */
+#define	EXF_DKIM	(1LL << 33)
 #endif /* _ACL_H_ */
