@@ -1,4 +1,4 @@
-/* $Id: milter-greylist.c,v 1.254 2013/01/27 02:08:00 manu Exp $ */
+/* $Id: milter-greylist.c,v 1.281 2016/01/31 05:34:55 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2012 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID  
-__RCSID("$Id: milter-greylist.c,v 1.254 2013/01/27 02:08:00 manu Exp $");
+__RCSID("$Id: milter-greylist.c,v 1.281 2016/01/31 05:34:55 manu Exp $");
 #endif
 #endif
 
@@ -90,6 +90,7 @@ static int check_drac(char *dotted_ip);
 #include "store.h"
 #include "stat.h"
 #include "milter-greylist.h"
+#include "brokenmfapi.h"
 #ifdef USE_DNSRBL
 #include "dnsrbl.h"
 #endif
@@ -102,9 +103,7 @@ static int check_drac(char *dotted_ip);
 #ifdef USE_DKIM
 #include "dkimcheck.h"
 #endif
-#if defined(USE_CURL) || defined(USE_LDAP)
 #include "prop.h"
-#endif
 #ifdef USE_GEOIP
 #include "geoip.h"
 #endif
@@ -113,11 +112,18 @@ static int check_drac(char *dotted_ip);
 #endif
 #include "macro.h"
 #include "ratelimit.h"
+#ifdef USE_NSUPDATE
+#include "nsupdate.h"
+#endif
 
 #ifdef USE_DMALLOC
 #include <dmalloc.h> 
 #endif
 
+#ifndef PACKAGE_URL
+#define PACKAGE_URL ""
+#endif
+
 static char *gmtoffset(time_t *, char *, size_t);
 static void writepid(char *);
 static void log_and_report_greylisting(SMFICTX *, struct mlfi_priv *, char *);
@@ -131,6 +137,7 @@ static sfsistat stat_from_code(char *);
 static void cleanup_pidfile(char *);
 static void cleanup_sock(char *);
 static int mg_setreply(SMFICTX *, struct mlfi_priv *, char *);
+static int resolve_bracketed(char *addr);
 static sfsistat real_connect(SMFICTX *, char *, _SOCK_ADDR *);
 static sfsistat real_helo(SMFICTX *, char *);
 static sfsistat real_envfrom(SMFICTX *, char **);
@@ -368,6 +375,8 @@ real_connect(ctx, hostname, addr)
 	TAILQ_INIT(&priv->priv_body);
 	priv->priv_maxpeek = conf.c_maxpeek;
 	priv->priv_msgcount = 0;
+	priv->priv_peekcount = 0;
+	priv->priv_peekdone = 0;
 	priv->priv_buf = NULL;
 	priv->priv_buflen = 0;
 	priv->priv_max_elapsed = 0;
@@ -376,6 +385,9 @@ real_connect(ctx, hostname, addr)
 	strncpy(priv->priv_hostname, hostname, ADDRLEN);
 	priv->priv_hostname[ADDRLEN] = '\0';
 
+	if ((priv->priv_hostname[0] == '[') && conf.c_unbracket)
+		(void)resolve_bracketed(priv->priv_hostname);
+
 	if (addr != NULL) {
 		switch (addr->sa_family) {
 		case AF_INET:
@@ -412,18 +424,24 @@ real_connect(ctx, hostname, addr)
 #ifdef USE_GEOIP
 	geoip_set_ccode(priv);
 #endif
+#ifdef HAVE_SPF2
+	priv->priv_spf_header = NULL;
+#endif
+#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
+     defined(HAVE_SPF2_10) || defined(HAVE_SPF2))
+	priv->priv_spf_result[0] = '\0';
+#endif
 #ifdef USE_DKIM
 	priv->priv_dkim = NULL;
 	priv->priv_dkimstat = DKIM_STAT_OK;
+	priv->priv_dkim_result[0] = '\0';
 #endif
 #ifdef USE_P0F
 	priv->priv_p0f = NULL;
 	p0f_lookup(priv);
 #endif
-#if defined(USE_CURL) || defined(USE_LDAP)
 	LIST_INIT(&priv->priv_prop);
 	priv->priv_prop_match = NULL;
-#endif /* USE_CURL || USE_LDAP */
 	priv->priv_max_tarpitted = 0;
 	priv->priv_total_tarpitted = 0;
 
@@ -488,26 +506,13 @@ real_envfrom(ctx, envfrom)
 	if (priv->priv_buf)
 		free(priv->priv_buf);
 	priv->priv_msgcount = 0;
+	priv->priv_peekcount = 0;
+	priv->priv_peekdone = 0;
 
 #ifdef USE_SPAMD
 	priv->priv_spamd_flags = 0;
 #endif
 
-	/* 
-	 * Now let's handle this new message...
-	 */
-	if ((priv->priv_queueid = smfi_getsymval(ctx, "{i}")) == NULL) {
-#ifndef USE_POSTFIX
-		/* 
-		 * Postfix does not choose a queue file name 
-		 * until after it accepts the first valid RCPT TO 
-		 * command, so don't log the failure 
-		 */
-		mg_log(LOG_DEBUG, "smfi_getsymval failed for {i}");
-#endif
-		priv->priv_queueid = "(unknown id)";
-	}
-
 	/*
 	 * Strip spaces from the source address
 	 */
@@ -573,17 +578,6 @@ real_envfrom(ctx, envfrom)
 	 * Is the sender address SPF-compliant?
 	 */
 	if ((conf.c_nospf == 0) && SPF_CHECK(priv)) {
-		char ipstr[IPADDRSTRLEN];
-
-		if (iptostring(SA(&priv->priv_addr),
-		    priv->priv_addrlen, ipstr, sizeof(ipstr))) {
-
-			mg_log(LOG_INFO,
-			    "%s: Sender IP %s and address %s are "
-			    "SPF-compliant, bypassing greylist", 
-			    priv->priv_queueid, ipstr, *envfrom);
-		}
-
 		priv->priv_sr.sr_elapsed = 0;
 		priv->priv_sr.sr_whitelist = EXF_WHITELIST | EXF_SPF;
 
@@ -616,6 +610,19 @@ real_envrcpt(ctx, envrcpt)
 		mg_log(LOG_ERR, "Internal error: smfi_getpriv() returns NULL");
 		return SMFIS_TEMPFAIL;
 	}
+	if ((priv->priv_queueid = smfi_getsymval(ctx, "{i}")) == NULL) {
+#ifndef USE_POSTFIX
+		/* 
+		 * Postfix does not choose a queue file name 
+		 * until after it accepts the first valid RCPT TO 
+		 * command, unless config option
+		 * 'smtpd_delay_open_until_valid_rcpt = no' set,
+		 * so don't log the failure.
+		 */
+		mg_log(LOG_DEBUG, "smfi_getsymval failed for {i}");
+#endif
+		priv->priv_queueid = "(unknown id)";
+	}
 	if (tarpit_reentry(priv) == SMFIS_TEMPFAIL)
 		return SMFIS_TEMPFAIL;
 
@@ -628,17 +635,50 @@ real_envrcpt(ctx, envrcpt)
 		    priv->priv_queueid, priv->priv_hostname, 
 		    addrstr, priv->priv_from, *envrcpt);
 
-#if defined(USE_CURL) || defined(USE_LDAP)
 	/*
 	 * Avoid properties gathered by urlcheck 
 	 * to mix for multiple recipients.
 	 */
 	prop_clear(priv, UP_CLEARPROP);
-#endif
 
-	if ((priv->priv_sr.sr_whitelist & EXF_WHITELIST) &&
-	    (priv->priv_sr.sr_whitelist &
-	     (EXF_NONIP | EXF_AUTH | EXF_STARTTLS | EXF_SPF)))
+	/*
+	 * If we re-evaluate racl for each recipient, forget
+	 * about previous decision. 
+	 */
+	if (conf.c_multiracl)
+		priv->priv_sr.sr_whitelist &=
+		     ~(EXF_WHITELIST|EXF_GREYLIST|EXF_BLACKLIST); 
+
+	/*
+	 * Global authenticated or TLS whitelisting, unless
+	 * - noauth global parameter is set, or
+	 * - any ACL with auth or tls clause
+	 */
+	if (conf.c_noauth == 0) {
+		if (priv->priv_sr.sr_whitelist & EXF_AUTH)
+			goto exit_accept;
+		if (priv->priv_sr.sr_whitelist & EXF_STARTTLS)
+			goto exit_accept;
+	}
+
+	/*
+	 * Global SPF whitelisting, unless
+	 * - nospf global parameter is set, or
+	 * - any ACL with spf clause
+	 */
+	if ((conf.c_nospf == 0) && (priv->priv_sr.sr_whitelist & EXF_SPF)) {
+		if (priv->priv_rcptcount == 0) // Log only for first recipient
+			mg_log(LOG_INFO,
+				"%s: Sender IP %s and address %s are "
+				"SPF-compliant, bypassing greylist", 
+				priv->priv_queueid, addrstr, priv->priv_from);
+		goto exit_accept;
+	}
+
+	/*
+	 * Mail sent from non IP source is always whitelisted
+	 */
+	if (priv->priv_sr.sr_whitelist & EXF_NONIP) 
 		goto exit_accept;
 
 #ifdef USE_DRAC
@@ -892,6 +932,9 @@ real_body(ctx, chunk, size)
 	struct mlfi_priv *priv;
 	struct line *l;
 	size_t linelen;
+	size_t tsize;	/* data already saved plus last incomplete line */
+	size_t bsize;	/* to be copied from saved data in priv_buf */
+	size_t csize;	/* to be copied from current chunk */
 	int i;
 #ifdef USE_DKIM
 	sfsistat stat = SMFIS_CONTINUE;
@@ -902,27 +945,15 @@ real_body(ctx, chunk, size)
 		return SMFIS_TEMPFAIL;
 	}
 
+	priv->priv_msgcount += size;
+
 #ifdef USE_DKIM
 	if ((stat = dkimcheck_body(chunk, size, priv)) != SMFIS_CONTINUE)
 		return stat;
 #endif
 
-	priv->priv_msgcount += size;
-
-	/* Avoid copying the whole message to save CPU */
-	if (priv->priv_maxpeek == 0)
-		return SMFIS_CONTINUE;
-
-	if ((priv->priv_msgcount > priv->priv_maxpeek) || 
-	    (priv->priv_buflen > priv->priv_maxpeek)) {
-		mg_log(LOG_DEBUG, "ignoring message beyond maxpeek = %d", 
-		    priv->priv_maxpeek);
-		priv->priv_maxpeek = 0;
-		return SMFIS_CONTINUE;
-	}
-
 	/* First time: add \r\n between headers and body */
-	if (TAILQ_EMPTY(&priv->priv_body) && (priv->priv_buflen == 0)) {
+	if (TAILQ_EMPTY(&priv->priv_body)) {
 		const char crlf[] = "\r\n";
 
 		if ((l = malloc(sizeof(*l))) == NULL) {
@@ -939,52 +970,112 @@ real_body(ctx, chunk, size)
 		TAILQ_INSERT_TAIL(&priv->priv_body, l, l_list);
 
 		priv->priv_msgcount += strlen(crlf);
+		priv->priv_peekcount += strlen(crlf);
 	}
 
-	for (i = size - 1; i >= 0; i--) {
+	/*
+	 * maxpeek limit the length of data we inspect.
+	 * priv->priv_peekcount count the data seen so far.
+	 * If we already reached the limit, jusy continue now.
+	 */
+	if (priv->priv_peekcount >= priv->priv_maxpeek) {
+		if (!priv->priv_peekdone) {
+			mg_log(LOG_DEBUG, 
+			       "ignoring message beyond maxpeek = %d", 
+ 			       priv->priv_maxpeek);
+			priv->priv_peekdone = 1;
+		}
+		return SMFIS_CONTINUE;
+	}
+
+	/*
+	 * priv->priv_buflen count a possible incomplete line
+	 * we got from previous call. If it is enough to reach
+	 * the limit, we do not take any data from the new chunk,
+	 * and we also disregard the incmplete saved line.
+	 */
+	tsize = priv->priv_peekcount + priv->priv_buflen;
+	if (tsize > priv->priv_maxpeek) {
+		bsize = priv->priv_maxpeek - priv->priv_peekcount;
+		csize = 0;
+	} else {
+		/* 
+		 * If saved data plus current chunk reaches the
+		 * limit, use all saved data and truncate the chunk.
+		 * Otherwise, take everythin.
+		 */
+		if (tsize + size > priv->priv_maxpeek) {
+			bsize = priv->priv_buflen;
+			csize = priv->priv_maxpeek - tsize;
+		} else {
+			bsize = priv->priv_buflen;
+			csize = size;
+		}
+	} 
+
+	/* 
+	 * No data from new chunk: we are done
+	 */
+	if (csize == 0) {
+		priv->priv_peekcount += bsize + csize; 
+		return SMFIS_CONTINUE;
+	}
+
+	/*
+	 * a struct line can hold multiple lines from the message,
+	 * We lookup the last \n in the chunk to save the unfinished
+	 * line for later.
+	 */
+	for (i = csize - 1; i >= 0; i--) {
 		if (chunk[i] == '\n')
 			break;
 	}
 	++i; /* Use i as byte counter */
 
+	/* From chunk to chunk + i - 1: a set of lines */
 	if (chunk[i - 1] == '\n') { /* We have a newline */
 		if ((l = malloc(sizeof(*l))) == NULL) {
 			mg_log(LOG_ERR, "malloc() failed: %s", strerror(errno));
 			exit(EX_OSERR);
 		}
 	
-		linelen = priv->priv_buflen + i;
-
+		/* Make room for data saved from previous call */
+		linelen = bsize + i;
 		if ((l->l_line = malloc(linelen + 1)) == NULL) {
 			mg_log(LOG_ERR, "malloc() failed: %s", strerror(errno));
 			exit(EX_OSERR);
 		}
 
-		/* Gather data saved from a previous call */
-		if (priv->priv_buf) {
-			memcpy(l->l_line, priv->priv_buf, priv->priv_buflen);
+		/* Gather data saved from a previous call, then the chunk */
+		if (bsize != 0) {
+			memcpy(l->l_line, priv->priv_buf, bsize);
 			free(priv->priv_buf);
 			priv->priv_buf = NULL;
+			priv->priv_buflen = 0;
 		}
 
-		memcpy(l->l_line + priv->priv_buflen, chunk, i);
+		memcpy(l->l_line + bsize, chunk, i);
 		l->l_line[linelen] = '\0';
 		l->l_len = linelen;
-		priv->priv_buflen = 0;
 
 		TAILQ_INSERT_TAIL(&priv->priv_body, l, l_list);
+
+		priv->priv_peekcount += linelen;
 	}
 
-	if(i < size) { /* keep the remains for later */
-		if ((priv->priv_buf = realloc(priv->priv_buf, 
-		    priv->priv_buflen + size - i)) == NULL) {
+	/* From chunk + i to chunk + csize - 1: an incomplete line */
+	if (i < csize) { /* keep the remains for later */
+		priv->priv_buf = realloc(priv->priv_buf, 
+					 priv->priv_buflen + csize - i);
+		if (priv->priv_buf == NULL) {
 			mg_log(LOG_ERR, 
 			    "realloc() failed: %s", 
 			    strerror(errno));
 			exit(EX_OSERR);
 		}
-		memcpy(&priv->priv_buf[priv->priv_buflen], chunk + i, size - i);
-		priv->priv_buflen += size - i;
+		memcpy(&priv->priv_buf[priv->priv_buflen], 
+		       chunk + i, csize - i);
+		priv->priv_buflen += (csize - i);
 	}
 
 	return SMFIS_CONTINUE;
@@ -1083,6 +1174,10 @@ real_eom(ctx)
 				priv->priv_from, aclstr);
 		}
 
+	/*
+	 * If we re-evaluate racl for each recipient, forget
+	 * about whitelisted addresses
+	 */
 		set_sr_defaults(priv, code, ecode, msg);
 		mg_setreply(ctx, priv, NULL);
 		free(aclstr);
@@ -1151,7 +1246,12 @@ passed:
 		} else {
 			*hdrvalue = '\0';
 			hdrvalue += strlen(sep);
-			smfi_addheader(ctx, hdrname, hdrvalue);
+			if (priv->priv_sr.sr_addheader_index >= 0)
+				smfi_insheader(
+					ctx, priv->priv_sr.sr_addheader_index,
+					hdrname, hdrvalue);
+			else
+				smfi_addheader(ctx, hdrname, hdrvalue);
 		}
 
 		free(hdrname);
@@ -1290,6 +1390,10 @@ passed:
 				ADD_REASON(whystr, "Sender passed SPF test");
 				priv->priv_last_whitelist &= ~EXF_SPF;
 			}
+			if (priv->priv_last_whitelist & EXF_DKIM) {
+				ADD_REASON(whystr, "Sender passed DKIM test");
+				priv->priv_last_whitelist &= ~EXF_DKIM;
+			}
 			if (priv->priv_last_whitelist & EXF_NONIP) {
 #ifdef AF_INET6
 				ADD_REASON(whystr, 
@@ -1390,7 +1494,12 @@ out:
 		} else {
 			*hdrvalue = '\0';
 			hdrvalue += strlen(sep);
-			smfi_addheader(ctx, hdrname, hdrvalue);
+			if (priv->priv_sr.sr_addheader_index >= 0)
+				smfi_insheader(
+					ctx, priv->priv_sr.sr_addheader_index,
+					hdrname, hdrvalue);
+			else
+				smfi_addheader(ctx, hdrname, hdrvalue);
 		}
 
 		free(hdrname);
@@ -1452,9 +1561,7 @@ real_close(ctx)
 		}
 		if (priv->priv_buf)
 			free(priv->priv_buf);
-#if defined(USE_CURL) || defined(USE_LDAP)
 		prop_clear(priv, UP_PLAINPROP);
-#endif
 #ifdef USE_DNSRBL
 		dnsrbl_list_cleanup(priv);
 #endif     
@@ -1462,6 +1569,10 @@ real_close(ctx)
 		if (priv->priv_p0f != NULL)
 			free(priv->priv_p0f);
 #endif
+#ifdef HAVE_SPF2
+		if (priv->priv_spf_header != NULL)
+			free(priv->priv_spf_header);
+#endif
 #ifdef USE_DKIM
 		dkimcheck_free(priv);
 #endif
@@ -1546,6 +1657,13 @@ main(argc, argv)
 		case 'r':
 			mg_log(LOG_INFO, "milter-greylist-%s %s", 
 			    PACKAGE_VERSION, BUILD_ENV);
+			mg_log(LOG_INFO,
+			    "Version: %s%s%s",
+			    PACKAGE_STRING,
+			    strlen(PACKAGE_URL) ? 
+				" distributed from " PACKAGE_URL : "" ,
+			    strlen(PACKAGE_BUGREPORT) ? 
+				", report bugs to " PACKAGE_BUGREPORT : "" );
 			exit(EX_OK);
 			break;
 
@@ -1731,6 +1849,9 @@ main(argc, argv)
 #endif
 	macro_init();
 	ratelimit_init();
+#ifdef USE_NSUPDATE
+	nsupdate_init();
+#endif
 
 #ifdef USE_FD_POOL
         /* initialize file descriptor pool */
@@ -1890,7 +2011,13 @@ main(argc, argv)
 				exit(EX_OSERR);
 			}
 			pw->pw_gid = gr->gr_gid;
-		}
+		} else {
+			if ((gr = getgrgid(pw->pw_gid)) == NULL) {
+				mg_log(LOG_ERR, "%s: cannot get group %d data: %s",
+			    	argv[0], pw->pw_gid, strerror(errno));
+				exit(EX_OSERR);
+			}
+                }
 
 #ifdef HAVE_INITGROUPS
 		if (initgroups(conf.c_user, pw->pw_gid) != 0) {
@@ -1916,7 +2043,8 @@ main(argc, argv)
  		/* 
  		 * chown the socket so that sendmail does not complain
  		 */
-		if (chown(conf.c_socket, pw->pw_uid, pw->pw_gid) != 0)
+		if ((access(conf.c_socket, F_OK) == 0) &&
+		    (chown(conf.c_socket, pw->pw_uid, pw->pw_gid) != 0))
 			mg_log(LOG_WARNING, "%s: cannot change \"%s\""
 			    " ownership to %s/%s: %s", argv[0], 
 			    conf.c_socket, pw->pw_name, gr->gr_name,
@@ -1979,6 +2107,13 @@ void
 usage(progname)
 	char *progname;
 {
+	mg_log(LOG_ERR,
+	    "Version: %s%s%s",
+	    PACKAGE_STRING,
+	    strlen(PACKAGE_URL) ?
+		" distributed from " PACKAGE_URL : "" ,
+	    strlen(PACKAGE_BUGREPORT) ?
+		", report bugs to " PACKAGE_BUGREPORT : "" );
 	mg_log(LOG_ERR,
 	    "usage: %s [-A] [-a autowhite_delay] [-c] [-D] [-d dumpfile]",
 	    progname);
@@ -1987,7 +2122,7 @@ usage(progname)
 	mg_log(LOG_ERR,
 	    "       [-u username[:groupname]] [-v] [-w greylist_delay] [-L cidrmask]");
 	mg_log(LOG_ERR,
-	    "       [-M prefixlen] [-P pidfile] -p socket");
+	    "       [-M prefixlen] [-P pidfile] [-p socket]");
 	exit(EX_USAGE);
 }
 
@@ -2327,7 +2462,7 @@ check_drac(dotted_ip)
 {
 	DB *ddb;
 	DBT key, data;
-	char ipkey[16];
+	char ipkey[64];
 	int rc;
 
 	ddb = dbopen(conf.c_dracdb, O_RDONLY | O_SHLOCK, 0666, DB_BTREE, NULL);
@@ -2337,6 +2472,7 @@ check_drac(dotted_ip)
 	}
 
 	key.data = strncpy(ipkey, dotted_ip, sizeof(ipkey));
+	ipkey[sizeof(ipkey) - 1] = '\0';
 	key.size = strlen(ipkey);
 	rc = ddb->get(ddb, &key, &data, 0);
 	ddb->close(ddb);
@@ -2650,6 +2786,17 @@ domain_only(out, in, len)
 	return outp;
 }
 
+/*
+ * fstring_expand had initially been introduced to craft URL.
+ * outmaxlen was therefore set to URLMAXLEN, that is 255. 
+ * It evolved into a ubuquitous tool used everywhere in 
+ * milter-greylist sources, with the need of longer output
+ * strings. outmaxlen is not initialized to QSTRLEN, which
+ * has been increased to 4096. This means it is possible 
+ * to output longer URL. This could cause trouble, but 
+ * it does not violate any standard, therefore we live with it.
+ */
+
 char *
 fstring_expand(priv, rcpt, fstring, cv)
 	struct mlfi_priv *priv;
@@ -2659,10 +2806,10 @@ fstring_expand(priv, rcpt, fstring, cv)
 {
 	size_t offset;
 	char *outstr;
-	size_t outmaxlen = URLMAXLEN;
+	size_t outmaxlen = QSTRLEN;
 	char *tmpstr;
 	char *tmpstrp;
-	char *last;
+	char *last = NULL;
 	char *ptok;
 	int fstr_len;	/* format string length, minus the % (eg: %mr -> 2) */
 	int skip_until_brace_close = 0;
@@ -2678,6 +2825,16 @@ fstring_expand(priv, rcpt, fstring, cv)
 		return outstr;
 	}
 		
+	/*
+	 * The rcpt argument is not really mandatory since we can
+	 * figure out what the recipient is. The operation below
+	 * catches the numerous location where fstring_expand()
+	 * was called with NULL rcpt. The rcpt argument should
+	 * probably be removed.
+	 */
+	if ((rcpt == NULL) && (priv->priv_cur_rcpt[0] != '\0'))
+		rcpt = priv->priv_cur_rcpt;
+
 	if ((outstr = malloc(outmaxlen + 1)) == NULL) {
 		mg_log(LOG_ERR, "malloc failed: %s", strerror(errno));
 		exit(EX_OSERR);
@@ -2752,11 +2909,12 @@ fstring_expand(priv, rcpt, fstring, cv)
 
 			switch(*(ptok + 1)) {
 			case 'r':	/* Recipient */
-				mystrncat(&outstr, 
-					mbox_only(tmpaddr, 
-					      rcpt, 
-					      ADDRLEN), 
-					&outmaxlen, cv);
+				if (rcpt != NULL)
+					mystrncat(&outstr, 
+						mbox_only(tmpaddr, 
+						      rcpt, 
+						      ADDRLEN), 
+						&outmaxlen, cv);
 				break;
 			case 'f':	/* Sender */
 				mystrncat(&outstr, 
@@ -2783,11 +2941,12 @@ fstring_expand(priv, rcpt, fstring, cv)
 
 			switch(*(ptok + 1)) {
 			case 'r':	/* Recipient */
-				mystrncat(&outstr, 
-					site_only(tmpaddr, 
-					      rcpt, 
-					      ADDRLEN), 
-					&outmaxlen, cv);
+				if (rcpt != NULL)
+					mystrncat(&outstr, 
+						site_only(tmpaddr, 
+						      rcpt, 
+						      ADDRLEN), 
+						&outmaxlen, cv);
 				break;
 			case 'f':	/* Sender */
 				mystrncat(&outstr, 
@@ -2816,6 +2975,30 @@ fstring_expand(priv, rcpt, fstring, cv)
 			mystrncat(&outstr, ipstr, &outmaxlen, cv);
 			break;
 		}
+		case 'j': {	/* Sender machine IP address, reversed */
+			char ipstr[IPADDRSTRLEN + 1];
+			char revipstr[IPADDRSTRLEN + 1];
+			char *lasts, *cp;
+			size_t len;
+
+			iptostring(SA(&priv->priv_addr),
+			    priv->priv_addrlen, ipstr, sizeof(ipstr));
+
+			len = strlen(ipstr);
+			(void)memset(revipstr, '.', len);
+			revipstr[len] = '\0';
+
+			/* XXX only works for IPv4? */
+			for (lasts = NULL, cp = strtok_r(ipstr, ".", &lasts);
+			     cp;
+			     cp = strtok_r(NULL, ".", &lasts)) {
+				size_t cplen = strlen(cp);
+				(void)memcpy(revipstr + len - cplen, cp, cplen);
+				len -= (cplen + 1); /* 1 for dot */
+			}
+			mystrncat(&outstr, revipstr, &outmaxlen, cv);
+			break;
+		}
 		case 'D': {
 #ifdef USE_DNSRBL
 			char dnsrbl[QSTRLEN + 1];
@@ -2956,7 +3139,104 @@ fstring_expand(priv, rcpt, fstring, cv)
 			mystrncat(&outstr, tzstr, &outmaxlen, cv);
 			break;
 		}
-		
+		case 'Q': { /* "Query" results */
+			fstr_len =  2;
+			switch(*(ptok + 1)) {
+			case 's': /* SPF result */
+#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
+     defined(HAVE_SPF2_10) || defined(HAVE_SPF2))
+				if (priv->priv_spf_result[0])
+					mystrncat(&outstr, 
+					  	priv->priv_spf_result,
+					  	&outmaxlen, cv);
+#else
+				fstr_len = 0;
+#endif
+				break;
+			case 'd': /* DKIM result */
+#ifdef USE_DKIM
+				if (priv->priv_dkim_result[0])
+					mystrncat(&outstr, 
+					  	priv->priv_dkim_result,
+					  	&outmaxlen, cv);
+#else
+				fstr_len = 0;
+#endif
+				break;
+			case 'A': { /* Authentication-Results header value */
+				char host[ADDRLEN + 1];
+				char output[HDRLEN + 1];
+				char *fqdn;
+				int cx;
+
+				fqdn = smfi_getsymval(priv->priv_ctx, "{j}");
+				if (fqdn == NULL) {
+					mg_log(LOG_DEBUG, 
+					  "smfi_getsymval failed for {j}");
+					gethostname(host, ADDRLEN);
+					fqdn = host;
+				}
+
+				cx = snprintf(output, HDRLEN, "%s", fqdn);
+
+#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
+     defined(HAVE_SPF2_10) || defined(HAVE_SPF2))
+				if ((cx < HDRLEN) && priv->priv_spf_result[0]) {
+					char tmpfrom[
+						strlen(priv->priv_from) - 1];
+
+					memcpy(tmpfrom,
+						priv->priv_from + 1,
+						sizeof(tmpfrom) - 1);
+					tmpfrom[sizeof(tmpfrom) - 1] = '\0';
+
+					cx += snprintf(output+cx, HDRLEN-cx,
+					      ";\r\n\tspf=%s smtp.mailfrom=%s",
+					      priv->priv_spf_result, tmpfrom);
+				}
+#endif
+#ifdef USE_DKIM
+				if ((cx < HDRLEN) &&
+					priv->priv_dkim_result[0]) {
+					DKIM_SIGINFO *sig=NULL;
+					unsigned const char *domain;
+					cx += snprintf(output+cx, HDRLEN-cx,
+						";\r\n\tdkim=%s",
+						priv->priv_dkim_result);
+
+					if (priv->priv_dkim != NULL)
+						sig = dkim_getsignature(
+							priv->priv_dkim);
+					if ((cx < HDRLEN) && (sig != NULL) &&
+					    ((domain = dkim_sig_getdomain(sig))
+					     != NULL))
+						cx += snprintf(output+cx,
+						HDRLEN-cx, " header.d=%s",
+						domain);
+				}
+#endif
+				if ((cx < HDRLEN) && (cx == strlen(fqdn)))
+					snprintf(output+cx, HDRLEN-cx,
+						"; none");
+				mystrncat(&outstr, output, &outmaxlen, cv);
+				break;
+			}
+			case 'S': /* Received-SPF header value */
+#ifdef HAVE_SPF2
+				if (priv->priv_spf_header != NULL)
+					mystrncat(&outstr, 
+					  	priv->priv_spf_header,
+					  	&outmaxlen, cv);
+#else
+				fstr_len = 0;
+#endif
+				break;
+			default:
+				fstr_len = 0;
+				break;
+			}
+			break;
+		}
 		case 'C': {	/* Country code from GeoIP */
 #ifdef USE_GEOIP
 			if (priv->priv_ccode != NULL)
@@ -3089,16 +3369,9 @@ fstring_expand(priv, rcpt, fstring, cv)
 			}
 
 			snprintf(output, HDRLEN, 
-#ifndef USE_POSTFIX
-			    "milter-greylist-%s (%s [%s]); %s %s (%s)",
-#else
-			    "milter-greylist-%s (%s); %s %s (%s)",
-#endif
-			    PACKAGE_VERSION, fqdn,
-#ifndef USE_POSTFIX
-			    local_ipstr(priv),
-#endif
-			    timestr, tzstr, tznamestr);
+				 "milter-greylist-%s (%s [%s]); %s %s (%s)",
+			         PACKAGE_VERSION, fqdn, local_ipstr(priv),
+				 timestr, tzstr, tznamestr);
 			mystrncat(&outstr, output, &outmaxlen, cv);
 			break;
 		}
@@ -3296,6 +3569,43 @@ fstring_expand(priv, rcpt, fstring, cv)
 			}
 			break;
 		}	
+		case 'c': {	/* currently evaluated ACL */
+			fstr_len = 2;
+
+			switch(*(ptok + 1)) {
+			case 'A': {	/* Line number for current ACL */
+				char buf[16];
+
+				if (priv->priv_acl) {
+					snprintf(buf, sizeof(buf), "%d", 
+					   priv->priv_acl->a_line); 
+					mystrncat(&outstr, buf, &outmaxlen, cv);
+				} else {
+					mystrncat(&outstr, "(none)", 
+						  &outmaxlen, cv);
+				}
+				break;
+			}	
+			case 'a': {	/* id string for current ACL */
+				char buf[QSTRLEN + 1];
+
+				if (priv->priv_acl && priv->priv_acl->a_id) {
+					snprintf(buf, sizeof(buf), "%s", 
+					   priv->priv_acl->a_id); 
+					mystrncat(&outstr, buf, &outmaxlen, cv);
+				} else if (priv->priv_acl) {
+					snprintf(buf, sizeof(buf), "%d", 
+					   priv->priv_acl->a_line); 
+					mystrncat(&outstr, buf, &outmaxlen, cv);
+				}
+				break;
+			}	
+			default:
+				fstr_len = 0;
+				break;
+			}
+			break;
+		}
 		case 't': {	/* tarpit time */
 			char buf[32];
 
@@ -3304,7 +3614,6 @@ fstring_expand(priv, rcpt, fstring, cv)
 			mystrncat(&outstr, buf, &outmaxlen, cv);
 			break;
 		}
-#if defined(USE_CURL) || defined(USE_LDAP)
 		case 'p': 	/* LDAP or CURL gathered props */
 			fstr_len = 2;
 
@@ -3380,7 +3689,6 @@ fstring_expand(priv, rcpt, fstring, cv)
 			free(name);
 			break;
 		}
-#endif
 		case '%':	/* Literal '%' */
 			mystrncat(&outstr, "%", &outmaxlen, cv);
 			break;
@@ -3543,16 +3851,15 @@ fstring_escape(fstring)
 	return fstring;
 }
 
-
-#ifndef USE_POSTFIX
 char *
 local_ipstr(priv)
 	struct mlfi_priv *priv;
 {
+	static int log_missing_if_addr = 1;
 	char *ip;
 
 	/* 
-	 * Macro {if_addr} does not exist in Postfix 
+	 * Macro {if_addr} does not exist in Postfix and CommSuite
 	 */
 	ip = smfi_getsymval(priv->priv_ctx, "{if_addr}");
 #ifdef AF_INET6
@@ -3570,13 +3877,20 @@ local_ipstr(priv)
 	}
 #endif /* AF_INET6 */
 	if (ip == NULL) {
-		mg_log(LOG_DEBUG, "smfi_getsymval failed for {if_addr}");
+		if (conf.c_localaddr_string[0] != '\0')
+			return conf.c_localaddr_string;
+
+		if (log_missing_if_addr) {
+			mg_log(LOG_DEBUG,
+			       "smfi_getsymval failed for {if_addr}, "
+			       "consider setting localaddr option");
+			log_missing_if_addr = 0;
+		}
 		ip = "0.0.0.0";
 	}
 
 	return ip;
 }
-#endif /* !USE_POSTFIX */
 
 static void
 set_sr_defaults(priv, code, ecode, msg)
@@ -3633,6 +3947,10 @@ stat_from_code(code)
 	return SMFIS_TEMPFAIL;
 }
 
+#ifndef MAXREPLYLEN
+#define MAXREPLYLEN 980
+#endif
+
 static int
 mg_setreply(ctx, priv, rcpt)
 	SMFICTX *ctx;
@@ -3640,6 +3958,7 @@ mg_setreply(ctx, priv, rcpt)
 	char *rcpt;
 {
 	int r;
+	size_t len;
 
 	if (priv->priv_sr.sr_msg_x != NULL) {
 		free(priv->priv_sr.sr_msg_x);
@@ -3648,9 +3967,111 @@ mg_setreply(ctx, priv, rcpt)
 
 	priv->priv_sr.sr_msg_x =
 		fstring_expand(priv, rcpt, priv->priv_sr.sr_msg, NULL);
-	r = smfi_setreply(ctx,
-			priv->priv_sr.sr_code, priv->priv_sr.sr_ecode,
-			priv->priv_sr.sr_msg_x);
+
+	if ((len = strlen(priv->priv_sr.sr_msg_x)) <= MAXREPLYLEN) {
+		r = smfi_setreply(ctx,
+				priv->priv_sr.sr_code, priv->priv_sr.sr_ecode,
+				priv->priv_sr.sr_msg_x);
+	} else {	/* multi line reply hell */
+		int lcount;
+		int i;
+		char **lbufs;
+		char *src;
+
+		lcount = len / MAXREPLYLEN;
+		if (len % MAXREPLYLEN != 0)
+			lcount++;
+
+		if ((lbufs = malloc(sizeof(*lbufs) * (lcount + 1))) == NULL) {
+			mg_log(LOG_ERR, "malloc failed: %s", strerror(errno));
+			exit(EX_OSERR);
+		}
+
+		src = priv->priv_sr.sr_msg_x;
+		for (i = 0; i < lcount; i++) {
+			if ((lbufs[i] = malloc(MAXREPLYLEN + 1)) == NULL) {
+				mg_log(LOG_ERR, "malloc failed: %s",
+				       strerror(errno));
+				exit(EX_OSERR);
+			}
+
+			(void)strncpy(lbufs[i], src, MAXREPLYLEN);
+			lbufs[i][MAXREPLYLEN] = 0;
+
+			src += MAXREPLYLEN;		
+		}
+
+		lbufs[i] = NULL;
+
+		r = mg_setmlreply(ctx, 
+				  priv->priv_sr.sr_code,
+				  priv->priv_sr.sr_ecode,
+				  lcount, lbufs);
+
+		for (i = 0; i < lcount; i++)
+			free(lbufs[i]);
+		free(lbufs);
+
+	}
+
 	return r;
 }
 
+
+static int
+resolve_bracketed(addr)
+	char *addr;
+{
+#if defined(HAVE_GETNAMEINFO) && defined(HAVE_GETADDRINFO)
+	size_t len;
+	struct addrinfo hint;
+	struct addrinfo *res;
+	char abuf[IPADDRSTRLEN];
+	int rc;
+
+	len = strlen(addr);
+	if ((len < 3) || (addr[0] != '[') || (addr[len - 1] != ']'))
+		return 0;
+
+	/*
+	 * Strip leading and trailing brackets. If the MTA
+	 * inserted a leading "IPv6:", also stip it.
+	 */
+	if (strncmp(addr, "[IPv6:", 6) == 0) {
+		(void)strncpy(abuf, addr + 6, len - 7);
+		abuf[len - 7] = '\0';
+	} else {
+		(void)strncpy(abuf, addr + 1, len - 2);
+		abuf[len - 2] = '\0';
+	}
+
+	(void)memset(&hint, 0, sizeof(hint));
+	hint.ai_flags = AI_NUMERICHOST;
+	
+	if ((rc = getaddrinfo(abuf, NULL, &hint, &res)) != 0) {
+		mg_log(LOG_ERR, "failed parsing %s: %s", 
+		     addr, gai_strerror(rc));		
+		return -1;
+	}
+           
+	rc = getnameinfo(res->ai_addr, res->ai_addrlen,
+			 abuf, IPADDRSTRLEN, NULL, 0, NI_NAMEREQD);
+	if (rc == 0) {
+		if (conf.c_debug)
+			mg_log(LOG_DEBUG, "Unbracketed '%s' into '%s'",
+			       addr, abuf);
+		(void)strncpy(addr, abuf, IPADDRSTRLEN);
+		addr[IPADDRSTRLEN] = '\0';
+	} else {
+		mg_log(LOG_INFO, "failed resolving %s: %s", 
+		       addr, gai_strerror(rc));		
+	}
+
+	freeaddrinfo(res);
+#else
+	mg_log(LOG_ERR, "unbracket option set whithout "
+	       "getaddrinfo(3) or getnameinfo(3) support";
+#endif /* HAVE_GETNAMEINFO && HAVE_GETADDRINFO */
+	return 0;
+}
+
