@@ -1,4 +1,4 @@
-/* $Id: milter-greylist.h,v 1.92 2012/09/20 08:31:49 manu Exp $ */
+/* $Id: milter-greylist.h,v 1.98 2015/10/05 04:18:34 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2012 Emmanuel Dreyfus
@@ -37,6 +37,10 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#include <libmilter/mfapi.h>
+#include "config.h"
+#include "dump.h"
+
 #ifdef USE_DKIM
 #ifdef HAVE_STDBOOL_H
 #include <stdbool.h>
@@ -44,15 +48,11 @@
 #include <dkim.h>
 #endif
 
-#include <libmilter/mfapi.h>
-#include "config.h"
-#include "dump.h"
-
 /* environment of Solaris workaround for stdio descriptor limitation */
 #include "fd_pool.h"
 
 #define NUMLEN 20
-#define QSTRLEN 1024
+#define QSTRLEN 4096
 #define REGEXLEN 1024
 #define HDRLEN 1024
 #define HEADERNAME "X-Greylist"
@@ -168,6 +168,7 @@ struct smtp_reply {
 	char *sr_report;
 	char *sr_report_x;
 	char *sr_addheader;
+	int sr_addheader_index;
 	char *sr_addfooter;
 	char *sr_subjtag;
 	sfsistat sr_retcode;
@@ -205,6 +206,8 @@ struct mlfi_priv {
 	const char *priv_ccode;
 #endif
 	size_t priv_msgcount;
+	size_t priv_peekcount;
+	int priv_peekdone;
 	char *priv_buf;
 	size_t priv_buflen;
 	char *priv_queueid;
@@ -212,16 +215,22 @@ struct mlfi_priv {
 	struct smtp_reply priv_sr;
 	time_t priv_max_elapsed;
 	long long priv_last_whitelist;
-#if defined(USE_CURL) || defined(USE_LDAP)
 	LIST_HEAD(, prop) priv_prop;
 	struct prop *priv_prop_match;
-#endif /* USE_CURL || USE_LDAP */
 #ifdef USE_DNSRBL
 	LIST_HEAD(, dnsrbl_list) priv_dnsrbl;
 #endif
+#ifdef HAVE_SPF2
+	char *priv_spf_header;
+#endif
+#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT) || \
+     defined(HAVE_SPF2_10) || defined(HAVE_SPF2))
+	char priv_spf_result[10];
+#endif
 #ifdef USE_DKIM
 	DKIM *priv_dkim;
 	DKIM_STAT priv_dkimstat;
+	char priv_dkim_result[10];
 #endif
 #ifdef USE_P0F
 	char *priv_p0f;
@@ -232,6 +241,7 @@ struct mlfi_priv {
 #endif
 	time_t priv_max_tarpitted;
 	time_t priv_total_tarpitted;
+	struct acl_entry *priv_acl;	/* current ACL being evaluated */
 };
 
 sfsistat mlfi_connect(SMFICTX *, char *, _SOCK_ADDR *);
@@ -249,9 +259,7 @@ sfsistat mlfi_data(SMFICTX *);
 #endif
 void usage(char *);
 int humanized_atoi(char *);
-#ifndef USE_POSTFIX
 char *local_ipstr(struct mlfi_priv *);
-#endif
 struct in_addr *prefix2mask4(int, struct in_addr *);
 #ifdef AF_INET6
 struct in6_addr *prefix2mask6(int, struct in6_addr *);
